##
# Printer.
# @author kaiba http://pokosho.com

use strict;
package Printer;
use util::Configure;
use Data::Dumper;
use util::SaveFileReader;
use util::DateTime;

##
# Constractor
sub new{
	my $this = shift;
	my $self = {
		# page index
		"index" => 0,
		# path to template file
		"template" => ""
	};
	bless $self, $this;
	return $self;
}

## 
# set index.
# @param index index
sub setIndex() {
	my $this = shift;
	my ($index) = @_ ;
	my $logger = Log::Log4perl::get_logger("mylogger");
	$logger->debug("setindex:$index");
	$this->{index} = $index;
}

##
# set template
# @param template path to template file.
sub setTemplate() {
	my $this = shift;
	my ($template) = @_;
	my $logger = Log::Log4perl::get_logger("mylogger");
	$logger->debug("template:$template");
	$this->{template} = $template;
}

##
# print page. 
sub print() {
	my $this = shift;
	my $html = $this->readTemplate();
	$html = $this->applyTemplate($html);
	print $html;
}

##
# read template. protected method.
# @param string replaced string.
# @return HTML apllied template.
sub readTemplate() {
	my $this = shift;
	my ($str) = @_;
	my $logger = Log::Log4perl::get_logger("mylogger");
	my $rc = open(TMPL, $this->{template});
	if ($rc == 0) {
		$logger->error("can't open template:$this->{template}");
		close(TMPL);
		return "";
	}
	my $line;
	my $html;
	while($line = <TMPL>) {
		$html = $html . $line;
	}
	close(TMPL);
	return $html;	
}

##
# apply template. protected method.
# @param string replaced string.
# @return HTML apllied template.
sub applyTemplate() {
	my $this = shift;
	my ($html) = @_;
	
	my $configure = Configure->getInstance();
	my $sfr = SaveFileReader->getInstance();
	my $printIndex = $this->{index} + 1;
	my $logger = Log::Log4perl::get_logger("mylogger");
	$logger->debug("start applyTemplate");
	$logger->debug("index:$this->{index} printIndex:$printIndex");
	# apply template
	my $s = $configure->getParam('TITLE');
	$html =~ s/<PAGE_TITLE>/$s/g;
	$s = $configure->getParam('DESC');
	$html =~ s/<PAGE_DESC>/$s/g;
	$html =~ s/<PAGE>/$printIndex/g;
	$s = $sfr->getLastUpdate();
	$html =~ s/<LAST_UPDATE>/$s/g;
	$s = $sfr->getLastCheckDate();
	$html =~ s/<LAST_CHECK_DATE>/$s/g;
	my $imgsref = $sfr->getImages();
	my @imgs = @$imgsref;
	my $start = $this->{index} * ($configure->getParam('PRINT_NUM'));

	# a image.
	$logger->debug("image index:$this->{index}");
	my $imgid = $this->{imageIndex};
	if (!defined $imgid){$imgid = 0;}
	my $hr = $imgs[$imgid];
	my %h = %$hr;
	$logger->debug("hash:".Dumper(%h));
	my @key = keys(%h);
	$logger->debug("index:$this->{index} key:$key[0]");
	$s = $configure->getParam('UP_DIR')."/".$key[0];
	$html =~ s/<IMAGE>/$s/g;
	$html =~ s/<IMAGE_NAME>/$key[0]/g;
	$s = $h{$key[0]};
	$html =~ s/<IMAGE_DATE>/$s/g;
	$s = $sfr->getImageNum();
	$html =~ s/<IMAGE_NUM>/$s/g;
	$s = $configure->getParam('SITE_URL');
	$html =~ s/<SITE_URL>/$s/g;
	my $dt = new DateTime();
	$s = $dt->convertW3CToRssDate($sfr->getLastCheckDate());
	$html =~ s/<LAST_CHECK_DATE_RSS>/$s/g;
	$s = $dt->convertW3CToRssDate($h{$key[0]});
	$html =~ s/<IMAGE_DATE_RSS>/$s/g;
	$s = $dt->convertW3CToRssDate($sfr->getLastUpdate());
	$html =~ s/<LAST_UPDATE_RSS>/$s/g;
	$s = $configure->getParam('SITE_URL').$configure->getParam('UP_DIR')."/".$key[0];
	$html =~ s/<IMAGE_URL>/$s/g;
	$s = $configure->getParam('TMPL_DIR');
	$html =~ s/<TMPL_DIR>/$s/g;
	$s = $configure->getParam('RSS_FILE');
	$html =~ s/<RSS_FILE>/$s/g;
	
	# pager(only next and prev)
	my $pagenum = $sfr->getImageNum() / $configure->getParam('PRINT_NUM');
	$logger->debug("pagenum:$pagenum index:$this->{index}");
	if ($this->{index} == 0) {
		$logger->debug("top page");
		$html =~ s/<PAGER_PREV>/index.cgi#/g;
	} else {
		$s = $this->{index} - 1;
		$html =~ s/<PAGER_PREV>/index.cgi\?index=$s/g;
	}
	if ($sfr->getImageNum() > $printIndex * $configure->getParam('PRINT_NUM') &&
			$sfr->getImageNum() > $configure->getParam('PRINT_NUM') &&
			$printIndex < $pagenum) {
		$s = $this->{index} + 1;
	} else {
		$logger->debug("last page");
		$s = $this->{index};
	}
	$html =~ s/<PAGER_NEXT>/index.cgi\?index=$s/g;
	
	# print pager
	$this->printPager(\$html);
	return $html;
}

## print pager link. private method.
# @param html reference to thml string.
sub printPager() {
	my $this = shift;
	my ($htmlref) = @_;
	my $html = $$htmlref;
	my $configure = Configure->getInstance();
	my $sfr = SaveFileReader->getInstance();
	my $s;
	my $printIndex = $this->{index} + 1;
	my $pagerTmpl = "";
	my $pagerHtml = "";
	my $logger = Log::Log4perl::get_logger("mylogger");
	# start print pager
	my $pagenum = $sfr->getImageNum() / $configure->getParam('PRINT_NUM');
	if ($this->{index} == 0) {
		$html =~ s/<PAGER_PREV>/index.cgi#/g;
	} else {
		$s = $this->{index} - 1;
		$html =~ s/<PAGER_PREV>/index.cgi\?index=$s/g;
	}
	if ($sfr->getImageNum() > $printIndex * $configure->getParam('PRINT_NUM') &&
			$sfr->getImageNum() > $configure->getParam('PRINT_NUM') &&
			$printIndex < $pagenum) {
		$s = $this->{index} + 1;
		$html =~ s/<PAGER_NEXT>/index.cgi\?index=$s/g;
	} else {
		$html =~ s/<PAGER_NEXT>/index.cgi#/g;
	}
	$html =~ m/<PAGER_START>(.+)<PAGER_END>/sg;
	$pagerTmpl = $1;
	for (my $i = 0; defined $pagerTmpl && $i < $pagenum; $i++) {
		$s = $pagerTmpl;
		my $pagerText = $i + 1;
		my $ps = "index.cgi?index=$i";
		$logger->debug("pager link param:$ps");
		if ($i == $this->{index}) {
			$s =~ s/<PAGER_LINK>/index.cgi#/g;
			$s =~ s/<PAGER_NUM>/$pagerText/g;
		} else {
			$s =~ s/<PAGER_LINK>/$ps/g;
			$s =~ s/<PAGER_NUM>/$pagerText/g;
		}
		$logger->debug("pager link html:$s");
		$pagerHtml = $pagerHtml . $s;
	}
	$html =~ s/<PAGER_START>.+<PAGER_END>/$pagerHtml/sg;
	$$htmlref = $html;
	
}

1;
