##
# Updater. Create new update file and rss file.
# @author kaiba http://pokosho.com

package Updater;
use request::Requester;
use base qw(Requester);
use strict;
use util::Configure;
use Data::Dumper;
use util::SaveFileReader;

##
# Constractor
sub new() {
	my $this = shift;
	my $self = {};
	bless $self, $this;
	return $self;
}

## request. 
# @param index index
# @param opt optional string
sub request() {
	my $this = shift;
	my ($index, $opt) = @_ ;
	my $configure = Configure->getInstance();
	my $sfr = SaveFileReader->getInstance();
	my $imgsref = $sfr->getImages();
	my $logger = Log::Log4perl::get_logger("mylogger");
	my @imgs;
	my @newImgs;
	my @delImgs;
	my $s;
	my $fs;
	my $i;
	my @fnames;
	$this->{index} = $index;
	$this->{opt} = $opt;
	$logger->debug("start update.");
	# get file names from save file.
	@imgs = @{$sfr->getImageFileNames()};
	
	# get current images in up dir.
	$s = $configure->getParam('UP_DIR');
	my $sf = $configure->getParam('SAVE_FILE');
	$fs = `ls -t1 $s | grep "[^$sf]"`;
	@fnames = split(/\n/, $fs);
	
	# check new images.
	@newImgs = @{$this->getNewImages(\@imgs,\@fnames)};
	
	# check deleted images.
	@delImgs = @{$this->getDeletedImages(\@imgs,\@fnames)};
	
	$sfr->updateLastCheckDate();
	if ($#newImgs == -1 && $#delImgs == -1) {
		$logger->debug("Nothing todo. So update only LAST_CHECK_DATE.");
		$sfr->updateSaveFile();
		#$this->printResult();
		return;
	}
	
	# remove deleted file from save file
	for ($i = 0; $i < @delImgs; $i++) {
		$sfr->delImageInfo($delImgs[$i]);
	}
	# add new file to save file
	for ($i = 0; $i < @newImgs; $i++) {
		my %h;
		my @ii;
		$s = $configure->getParam('UP_DIR');
		@ii = stat("$s/$newImgs[$i]");
		my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($ii[9]);
		$year += 1900;
		$mon += 1;
		$s = $configure->getParam('TIMEZONE');
		$h{$newImgs[$i]} = sprintf("%04d/%02d/%02d %02d:%02d:%02d%s", 
			$year, $mon, $mday, $hour, $min, $sec, $s);
		$logger->debug("new hash:$newImgs[$i] -> $h{$newImgs[$i]}");
		$sfr->addNewImageInfo(\%h);
	}
	$sfr->updateImageNum();
	# dump new save file
	$sfr->updateLastUpdate();
	$sfr->updateSaveFile();
	#$this->printResult();
}

##
# get added image file names. Private method.
# @param pimgs past images(referense)
# @param cimgs current images(referense)
# @return added image file names(referense)
sub getNewImages() { 
	my $this = shift;
	my ($pimgs, $cimgs) = @_ ;
	my @pimgsa = @$pimgs;
	my @cimgsa = @$cimgs;
	my $i;
	my $j;
	my $f;
	my @nimgs;
	my $logger = Log::Log4perl::get_logger("mylogger");
	for ($i = 0; $i < @cimgsa; $i++) {
		$f = 0;
		for ($j = 0; $j < @pimgsa; $j++) {
			$logger->debug("[$cimgsa[$i]] vs [$pimgsa[$j]]");
			if ($cimgsa[$i] eq $pimgsa[$j]) {
				#$logger->debug("found:$cimgsa[$i]");
				$f = 1;
				last;
			}
		}
		if ($f == 0) {
			$logger->debug("push file:$cimgsa[$i]");
			push(@nimgs, $cimgsa[$i]);
		}
	}
	$logger->debug("added Images :".Dumper(@nimgs));
	return \@nimgs;
}

##
# get current images infomation(fila name and date). Private method.
# @param pimgs past images(referense)
# @param cimgs current images(referense)
# @return added image file names(referense)
sub getDeletedImages() {
	my $this = shift;
	my ($pimgs, $cimgs) = @_ ;
	my @pimgsa = @$pimgs;
	my @cimgsa = @$cimgs;
	my $i;
	my $j;
	my $f;
	my @dimgs;
	my $logger = Log::Log4perl::get_logger("mylogger");
	$logger->debug("start getDeletedImages");
	for ($i = 0; $i < @pimgsa; $i++) {
		$f = 0;
		for ($j = 0; $j < @cimgsa; $j++) {
			if ($pimgsa[$i] eq $cimgsa[$j]) {
				$f = 1;
				$logger->debug("found $cimgsa[$j]");
				last;
				
			}
		}
		if ($f == 0) {
			$logger->debug("del file:$cimgsa[$i]");
			push(@dimgs, $pimgsa[$i]);
		}
	}
	$logger->debug("deleted Images :".Dumper(@dimgs));
	return \@dimgs;
}

## print result for admin.
sub printResult() {
	my $this = shift;
	my $configure = Configure->getInstance();
	my $sfr = SaveFileReader->getInstance();
	my $logger = Log::Log4perl::get_logger("mylogger");
	my $printer = new Printer(0, 'update_result.tmpl');
	$printer->print();
}



1;
