##
# DateTime.
# @author kaiba http://pokosho.com

package DateTime;
use strict;
use Log::Log4perl;
use YAML::Tiny;
use Data::Dumper;

my @month = ('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec');
my @day = ('Sun','Mon','Tue','Wed','Thu','Fri','Sat');

##
# Constructer
sub new() {
	my $this = shift;
	my $self = {};
	bless $self, $this;
	return $self;
}

## Convert W3C date string(ex: 2008/05/09 03:27:38+09:00) to 
# RFC822(ex: Sat, 07 Sep 2002 09:42:31 GMT)
# @param W3C date string 
sub convertW3CToRssDate() {
	my $this = shift;
	my ($w3c) = @_;
	my $logger = Log::Log4perl::get_logger("mylogger");
	$logger->debug("w3c:$w3c");
	$w3c =~ m/(\d{4})\/(\d{2})\/(\d{2}) (\d{2}):(\d{2}):(\d{2})(.)(\d{2}):(\d{2})/;
	my ($year, $mon, $mday, $hour, $min, $sec, $timezpm, $timez_hour, $timez_min);
	$year = $1;
	$mon = $2;
	$mday = $3;
	$hour = $4;
	$min = $5;
	$sec = $6;
	$timezpm = $7;
	$timez_hour = $8;
	$timez_min = $9;
	#$logger->debug("regexp check:$1 $2 $3 $4 $5 $6 $7 $8 $9");
	my $tmz = "$timezpm$timez_hour$timez_min";
	my $day = $this->getwday($year, $mon, $mday);
	$logger->debug("TMZ string:$tmz");
	my $s = sprintf("%s, %02d %s %4d %2d:%2d:%2d %s",
		$day, $mday, $month[$mon - 1],
		$year, $hour, $min, $sec, $tmz);
	return $s;
}

## Get day from year, month and date.
# @param year year
# @param mon month
# @param mday date
# @see http://easycgi.xrea.jp/perltips/zeller.htm
sub getwday {
	my $this = shift;
	my($year, $mon, $mday) = @_;
	my ($yr,$mt);
	if ($mon == 1 || $mon == 2) {
		$yr = $year;
		$yr--;
		$mt = $mon;
		$mt += 12;
	}
	else {
		$yr = $year;
		$mt = $mon;
	}
	my $i = ($yr + int($yr/4) - int($yr/100) + int($yr/400) + int((13*$mt + 8)/5) + $mday) %7;
	return $day[$i]
}

1;

