##
# SaveFileReader.
# @author kaiba http://pokosho.com

use strict;
package SaveFileReader;
use Log::Log4perl;
use YAML::Tiny;
use Data::Dumper;
use common::Const;
use Time::Local;

# Only one SaveFileReader obj.
my $sfr;

##
# get Configure obj.
sub getInstance{
	my $logger = Log::Log4perl::get_logger("mylogger");
	if(! $sfr) {
		my $this = shift;
		my $hash = {params => undef};
		my $filePath = {filePath => undef};
		#$sfr = bless {} => $class;
		bless $hash, $this;
		$sfr = $hash;
		$logger->debug("create new SaveFileReader.");
	}else{
		$logger->debug("return SaveFileReader.");
	}
	return $sfr;
}

##
# Load properties from configuration file.
# @param file File path of the save file.
sub load {
	my $this = shift;
	my ($file) = @_ ;
	my $logger = Log::Log4perl::get_logger("mylogger");
	$this->{filePath} = $file;
	my ($loaded);	
	if (-e $file) {
		$logger->debug("found SAVE_FILE:$file");
		$loaded = YAML::Tiny::LoadFile($file);
	} else {
		$logger->debug("not found SAVE_FILE:$file.");
		return Const::NG;
	}
	$logger->debug(Dumper($loaded));
	$this->{params} = $loaded;
	return Const::OK;
}

## Get IMAGE_NUM value.
# @return image num
sub getImageNum() {
	my $this = shift;
	return $this->{params}->{IMAGE_NUM};
}

## Get IMAGES. It's array of hashes.
# @return images hash
sub getImages() {
	my $this = shift;
	my $logger = Log::Log4perl::get_logger("mylogger");
	my @arr = $this->{params}->{IMAGES};
	#$logger->debug("getImages :".Dumper(@arr));
	return $this->{params}->{IMAGES};
}

## get LAST_UPDATE
# @return datetime
sub getLastUpdate() {
	my $this = shift;
	return $this->{params}->{LAST_UPDATE};
}

## Get LAST_CHECK_DATE
# @return datetime
sub getLastCheckDate() {
	my $this = shift;
	return $this->{params}->{LAST_CHECK_DATE};
}

##
# Get image names
# @return array of image names(reference).
sub getImageFileNames() {
	my $this = shift;
	my $imgsref = $this->getImages();
	my @imgs;
	my @imgsn;
	my $i;
	my $logger = Log::Log4perl::get_logger("mylogger");
	if (defined $imgsref) {
		@imgs = @$imgsref;
	}
	for ($i = 0; $i < @imgs; $i++) { 
		my $hr = $imgs[$i];
		my %h = %$hr;
		my @key = keys(%h);
		push(@imgsn, $key[0]);
	}
	#$logger->debug("file names in sf:".Dumper(@imgsn));
	return \@imgsn;
}

##
# Add new image infomation.
# @param h img hash.
sub addNewImageInfo() {
	my $this = shift;
	my ($hr) = @_;
	my %h = %$hr;
	my $logger = Log::Log4perl::get_logger("mylogger");
	#$logger->debug("arg:".Dumper(%h));
	
	$logger->debug("before addNewImageInfo:".Dumper($this->{params}));
	# add new image info to top of list.
	unshift(@{$this->{params}->{IMAGES}}, \%h);
	$logger->debug("after addNewImageInfo:".Dumper($this->{params}));
}

##
# Add new image infomation.
# @param s name of the image.
sub delImageInfo() {
	my $this = shift;
	my ($s) = @_;
	my $i;
	my $logger = Log::Log4perl::get_logger("mylogger");
	
	$logger->debug("before delImageInfo:".Dumper($this->{params}));


	my $imgsref = $this->getImages();
	my @imgs;
	my @imgsn;
	if ($imgsref != undef) {
		@imgs = @$imgsref;
	}
	for ($i = 0; $i < @imgs; $i++) { 
		my $hr = $imgs[$i];
		my %h = %$hr;
		my @key = keys(%h);
		if ($key[0] eq $s) {
			$logger->debug("del index:$i name:$s");
			splice(@{$this->{params}->{IMAGES}}, $i, 1);
		}
		push(@imgsn, $key[0]);
	}
	#$logger->debug("after delImageInfo:".Dumper($this->{params}));
}

##
# Update save file.
sub updateSaveFile() {
	my $this = shift;
	my $logger = Log::Log4perl::get_logger("mylogger");
	my $configure = Configure->getInstance();
	my $sf = $this->{filePath};
	$logger->debug("update savefile:".$sf);
	YAML::Tiny::DumpFile($sf, $this->{params});
}

##
# Update last check date.
sub updateLastCheckDate() {
	my $this = shift;
	$this->{params}->{LAST_CHECK_DATE} = $this->getCurrentDatetime();
}

##
# Update last check date.
sub updateLastUpdate() {
	my $this = shift;
	$this->{params}->{LAST_UPDATE} = $this->getCurrentDatetime();
}

##
# Update image num
sub updateImageNum() {
	my $this = shift;
	my $num = @_;
	my @ns = @{$sfr->getImageFileNames()};
	$this->{params}->{IMAGE_NUM} = $#ns + 1;
}

##
# Get current time without time zone as string.
sub getCurrentDatetime() {
	my $this = shift;
	my $configure = Configure->getInstance();
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime();
	$year += 1900;
	$mon += 1;
	my $s = $configure->getParam('TIMEZONE');
	return sprintf("%04d/%02d/%02d %02d:%02d:%02d%s", 
		$year, $mon, $mday, $hour, $min, $sec, $s);
}

## check it is to updated.
# @return TRUE: it it to updated.
sub isToUpcate() {
	my $this = shift;
	my $configure = Configure->getInstance();
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst);
	my $unixtime_b;
	my $unixtime_a;
	my $t;
	my $y;
	my $s = $this->getLastCheckDate();
	my $logger = Log::Log4perl::get_logger("mylogger");
	$logger->debug("getLastCheckDate:$s");
	$s =~ m/(\d{4})\/(\d{2})\/(\d{2}) (\d{2}):(\d{2}):(\d{2}).*/; # ex. 2008/04/29 13:06:07+09:00
	$logger->debug("regep check: $6,$5,$4,$3,$2,$1");
	$mon= $2 - 1;
	$y = $1 - 1900;
	$logger->debug("year:$y+1900 mon:$mon+1");
	$unixtime_b = timelocal($6,$5,$4,$3,$mon,$y);
	$unixtime_a = time();
	$t = $unixtime_a - $unixtime_b;
	$s = $configure->getParam('UPDATE_CHECK_INTERVAL');
	$s = $s * 60;
	#$logger->debug("$t vs $s");
	if ($t >= $s) {
		$logger->debug("time to update!");
		return Const::TRUE;
	}
	return Const::FALSE;
}

1;

